'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _rcUtil = require('rc-util');

var _rcUtil2 = _interopRequireDefault(_rcUtil);

var _Track = require('./Track');

var _Track2 = _interopRequireDefault(_Track);

var _Handle = require('./Handle');

var _Handle2 = _interopRequireDefault(_Handle);

var _Steps = require('./Steps');

var _Steps2 = _interopRequireDefault(_Steps);

var _Marks = require('./Marks');

var _Marks2 = _interopRequireDefault(_Marks);

function noop() {}

function pauseEvent(e) {
  e.cancelBubble = true;
  e.returnValue = false;
  if (e.stopPropagation) {
    e.stopPropagation();
  }
  if (e.preventDefault) {
    e.preventDefault();
  }
}

function getValueFromIndex(props) {
  var value = undefined;
  var marksLen = props.marks.length;
  var index = undefined;
  if ('index' in props) {
    index = props.index;
  } else {
    index = props.defaultIndex;
  }
  if (marksLen > 0) {
    value = (props.max - props.min) / (marksLen - 1) * index;
    value = value.toFixed(5) / 1;
  }
  return value;
}

var Slider = _react2['default'].createClass({
  displayName: 'Slider',

  propTypes: {
    min: _react2['default'].PropTypes.number,
    max: _react2['default'].PropTypes.number,
    step: _react2['default'].PropTypes.number,
    defaultValue: _react2['default'].PropTypes.number,
    defaultIndex: _react2['default'].PropTypes.number,
    value: _react2['default'].PropTypes.number,
    index: _react2['default'].PropTypes.number,
    marks: _react2['default'].PropTypes.array,
    isIncluded: _react2['default'].PropTypes.bool,
    className: _react2['default'].PropTypes.string,
    prefixCls: _react2['default'].PropTypes.string,
    disabled: _react2['default'].PropTypes.bool,
    children: _react2['default'].PropTypes.any,
    onBeforeChange: _react2['default'].PropTypes.func,
    onChange: _react2['default'].PropTypes.func,
    onAfterChange: _react2['default'].PropTypes.func,
    tipTransitionName: _react2['default'].PropTypes.string,
    withDots: _react2['default'].PropTypes.bool
  },

  getDefaultProps: function getDefaultProps() {
    return {
      min: 0,
      max: 100,
      step: 1,
      defaultValue: 0,
      marks: [],
      isIncluded: true,
      className: '',
      prefixCls: 'rc-slider',
      disabled: false,
      defaultIndex: 0,
      tipTransitionName: '',
      withDots: false
    };
  },

  getInitialState: function getInitialState() {
    var props = this.props;
    var value = props.defaultValue;
    if ('value' in props) {
      value = props.value;
    }
    value = this._trimAlignValue(value);
    var marksLen = props.marks.length;
    if (marksLen > 0) {
      value = getValueFromIndex(props);
    }
    return {
      dragging: false,
      value: value
    };
  },

  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    if ('value' in nextProps) {
      this.setState({
        value: nextProps.value
      });
    } else if ('index' in nextProps) {
      this.setState({
        value: getValueFromIndex(nextProps)
      });
    }
  },

  onMouseUp: function onMouseUp() {
    this._end('mouse');
  },

  onTouchUp: function onTouchUp() {
    this._end('touch');
  },

  onMouseMove: function onMouseMove(e) {
    var position = e.pageX || e.clientX + document.documentElement.scrollLeft; // to compat ie8
    this.onMove(e, position);
  },

  onTouchMove: function onTouchMove(e) {
    if (e.touches.length > 1 || e.type === 'touchend' && e.touches.length > 0) {
      this._end('touch');
      return;
    }

    var position = this._getTouchPosition(e);

    this.onMove(e, position);
  },

  onMove: function onMove(e, position) {
    pauseEvent(e);
    var props = this.props;
    var state = this.state;

    var value = state.value;
    var oldValue = value;

    var diffPosition = position - this.startPosition;

    var diffValue = diffPosition / this.getSliderLength() * (props.max - props.min);
    value = this._trimAlignValue(this.startValue + diffValue);

    if (value !== oldValue && !('value' in props) && !('index' in props)) {
      this.setState({ value: value });
    }
    if (value !== oldValue) {
      this._triggerEvents('onChange', value);
    }
  },

  onTouchStart: function onTouchStart(e) {
    if (e.touches.length > 1 || e.type.toLowerCase() === 'touchend' && e.touches.length > 0) {
      return;
    }

    var position = this._getTouchPosition(e);
    var value = this._calValueByPos(position);
    this._triggerEvents('onChange', value);
    this._start(position, value);
    this._addDocumentEvents('touch');
    pauseEvent(e);
  },

  onSliderMouseDown: function onSliderMouseDown(e) {
    var position = e.pageX || e.clientX + document.documentElement.scrollLeft; // to compat ie8
    var value = this._calValueByPos(position);
    this._triggerEvents('onChange', value);
    this._start(position, value);
    this._addDocumentEvents('mouse');
    pauseEvent(e);
  },

  getIndex: function getIndex(v) {
    var props = this.props;
    var value = v === undefined ? this.state.value : v;

    if (props.marks.length === 0) {
      return Math.floor((value - props.min) / props.step);
    }
    var unit = ((props.max - props.min) / (props.marks.length - 1)).toFixed(5);
    return Math.round(value / unit);
  },

  getSliderLength: function getSliderLength() {
    var slider = this.refs.slider;
    if (!slider) {
      return 0;
    }

    return slider.clientWidth;
  },

  getSliderStart: function getSliderStart() {
    var slider = this.refs.slider;
    var rect = slider.getBoundingClientRect();

    return rect.left;
  },

  render: function render() {
    var _rcUtil$classSet;

    var state = this.state;
    var value = state.value;
    var dragging = state.dragging;

    var props = this.props;
    var className = props.className;
    var prefixCls = props.prefixCls;
    var disabled = props.disabled;
    var isIncluded = props.isIncluded;
    var withDots = props.withDots;
    var marks = props.marks;
    var step = props.step;
    var max = props.max;
    var min = props.min;
    var tipTransitionName = props.tipTransitionName;
    var children = props.children;

    var marksLen = marks.length;

    var sliderClassName = _rcUtil2['default'].classSet((_rcUtil$classSet = {}, _defineProperty(_rcUtil$classSet, prefixCls, true), _defineProperty(_rcUtil$classSet, prefixCls + '-disabled', disabled), _defineProperty(_rcUtil$classSet, className, !!className), _rcUtil$classSet));

    var offset = this._calcOffset(value);

    var track = null;
    if (isIncluded) {
      var trackClassName = prefixCls + '-track';
      track = _react2['default'].createElement(_Track2['default'], { className: trackClassName, offset: 0, length: offset - 0 });
    }

    var handleClassName = prefixCls + '-handle';

    var steps = null;
    if (marksLen > 0 || step > 1 && withDots) {
      var stepsClassName = prefixCls + '-step';
      var stepNum = marksLen > 0 ? marksLen : Math.floor((max - min) / step) + 1;
      steps = _react2['default'].createElement(_Steps2['default'], { className: stepsClassName, stepNum: stepNum,
        index: this.getIndex(), isIncluded: isIncluded });
    }

    var mark = null;
    if (marksLen > 0) {
      var markClassName = prefixCls + '-mark';
      mark = _react2['default'].createElement(_Marks2['default'], { className: markClassName, marks: marks,
        index: this.getIndex(), isIncluded: isIncluded });
    }

    return _react2['default'].createElement(
      'div',
      { ref: 'slider', className: sliderClassName,
        onTouchStart: disabled ? noop : this.onTouchStart,
        onMouseDown: disabled ? noop : this.onSliderMouseDown },
      track,
      _react2['default'].createElement(_Handle2['default'], { className: handleClassName, prefixCls: prefixCls,
        offset: offset, tipTransitionName: tipTransitionName, value: value,
        dragging: dragging, noTip: marksLen > 0 }),
      steps,
      mark,
      children
    );
  },

  _trimAlignValue: function _trimAlignValue(v, propsArg) {
    var val = v;
    var props = propsArg || this.props;
    var step = props.marks.length > 0 ? (props.max - props.min) / (props.marks.length - 1) : props.step;

    if (val <= props.min) {
      val = props.min;
    }
    if (val >= props.max) {
      val = props.max;
    }

    var valModStep = (val - props.min) % step;
    var alignValue = val - valModStep;

    if (Math.abs(valModStep) * 2 >= step) {
      alignValue += valModStep > 0 ? step : -step;
    }

    return parseFloat(alignValue.toFixed(5));
  },

  _calcOffset: function _calcOffset(value) {
    var ratio = (value - this.props.min) / (this.props.max - this.props.min);
    return ratio * 100;
  },

  _calcValue: function _calcValue(offset) {
    var ratio = offset / this.getSliderLength();
    return ratio * (this.props.max - this.props.min) + this.props.min;
  },

  _calValueByPos: function _calValueByPos(position) {
    var pixelOffset = position - this.getSliderStart();
    // pixelOffset -= (this.state.onSize / 2);
    var nextValue = this._trimAlignValue(this._calcValue(pixelOffset));
    this.setState({
      value: nextValue
    });
    return nextValue;
  },

  _getTouchPosition: function _getTouchPosition(e) {
    var touch = e.touches[0];
    return touch.pageX;
  },

  _triggerEvents: function _triggerEvents(event, v) {
    var props = this.props;
    var hasMarks = props.marks && props.marks.length > 0;
    if (props[event]) {
      var data = undefined;
      if (hasMarks) {
        data = this.getIndex(v);
      } else if (v === undefined) {
        data = this.state.value;
      } else {
        data = v;
      }
      props[event](data);
    }
  },

  _addDocumentEvents: function _addDocumentEvents(type) {
    if (type === 'touch') {
      // just work for chrome iOS Safari and Android Browser
      this.onTouchMoveListener = _rcUtil.Dom.addEventListener(document, 'touchmove', this.onTouchMove);
      this.onTouchUpListener = _rcUtil.Dom.addEventListener(document, 'touchend', this.onTouchUp);
    } else if (type === 'mouse') {
      this.onMouseMoveListener = _rcUtil.Dom.addEventListener(document, 'mousemove', this.onMouseMove);
      this.onMouseUpListener = _rcUtil.Dom.addEventListener(document, 'mouseup', this.onMouseUp);
    }
  },

  _removeEventons: function _removeEventons(type) {
    if (type === 'touch') {
      this.onTouchMoveListener.remove();
      this.onTouchUpListener.remove();
    } else if (type === 'mouse') {
      this.onMouseMoveListener.remove();
      this.onMouseUpListener.remove();
    }
  },

  _start: function _start(position, value) {
    this._triggerEvents('onBeforeChange');
    this.startValue = value;
    this.startPosition = position;
    this.setState({
      dragging: true
    });
  },

  _end: function _end(type) {
    this._removeEventons(type);
    this._triggerEvents('onAfterChange');
    this.setState({
      dragging: false
    });
  }
});

exports['default'] = Slider;
module.exports = exports['default'];